/* SPDX-License-Identifier: GPL-2.0-only */


#ifndef LINUX_PATA_PARPORT_H
#define LINUX_PATA_PARPORT_H

#include <linux/libata.h>

#define PI_PCD	1	

struct pi_adapter {
	struct device dev;
	struct pi_protocol *proto;	
	int port;			
	int mode;			
	int delay;			
	int devtype;			
	char *device;			
	int unit;			
	int saved_r0;			
	int saved_r2;			
	unsigned long private;		
	struct pardevice *pardev;	
};

typedef struct pi_adapter PIA;	




#define delay_p			(pi->delay ? udelay(pi->delay) : (void)0)
#define out_p(offs, byte)	do { outb(byte, pi->port + offs); delay_p; } while (0)
#define in_p(offs)		(delay_p, inb(pi->port + offs))

#define w0(byte)		out_p(0, byte)
#define r0()			in_p(0)
#define w1(byte)		out_p(1, byte)
#define r1()			in_p(1)
#define w2(byte)		out_p(2, byte)
#define r2()			in_p(2)
#define w3(byte)		out_p(3, byte)
#define w4(byte)		out_p(4, byte)
#define r4()			in_p(4)
#define w4w(data)		do { outw(data, pi->port + 4); delay_p; } while (0)
#define w4l(data)		do { outl(data, pi->port + 4); delay_p; } while (0)
#define r4w()			(delay_p, inw(pi->port + 4))
#define r4l()			(delay_p, inl(pi->port + 4))

static inline u16 pi_swab16(char *b, int k)
{
	union { u16 u; char t[2]; } r;

	r.t[0] = b[2 * k + 1]; r.t[1] = b[2 * k];
	return r.u;
}

static inline u32 pi_swab32(char *b, int k)
{
	union { u32 u; char f[4]; } r;

	r.f[0] = b[4 * k + 1]; r.f[1] = b[4 * k];
	r.f[2] = b[4 * k + 3]; r.f[3] = b[4 * k + 2];
	return r.u;
}

struct pi_protocol {
	char name[8];

	int max_mode;
	int epp_first;		

	int default_delay;
	int max_units;		

	void (*write_regr)(struct pi_adapter *pi, int cont, int regr, int val);
	int (*read_regr)(struct pi_adapter *pi, int cont, int regr);
	void (*write_block)(struct pi_adapter *pi, char *buf, int count);
	void (*read_block)(struct pi_adapter *pi, char *buf, int count);

	void (*connect)(struct pi_adapter *pi);
	void (*disconnect)(struct pi_adapter *pi);

	int (*test_port)(struct pi_adapter *pi);
	int (*probe_unit)(struct pi_adapter *pi);
	int (*test_proto)(struct pi_adapter *pi, char *scratch, int verbose);
	void (*log_adapter)(struct pi_adapter *pi, char *scratch, int verbose);

	int (*init_proto)(struct pi_adapter *pi);
	void (*release_proto)(struct pi_adapter *pi);
	struct module *owner;
	struct device_driver driver;
	struct scsi_host_template sht;
};

#define PATA_PARPORT_SHT ATA_PIO_SHT

int pata_parport_register_driver(struct pi_protocol *pr);
void pata_parport_unregister_driver(struct pi_protocol *pr);

#define paride_register pata_parport_register_driver
#define paride_unregister pata_parport_unregister_driver

#endif 
